#ifndef TRACE_HPP
#define TRACE_HPP

#include <cmath>
#include <vector>
#include <cstdlib>
#include <stdexcept>

#define ONEDAY (8.64e4) // 24 * 3600
#define ONEYEAR (3.1536e9) // 365.25*ONEDAY

#define INF std::numeric_limits<double>::max()

class Trace
{
public:
	Trace(double mu = INF) : 
		mu(INF), last(0.0) {}

	double next(const double &t)
	{
		double next = last;

		while(t >= next)
		{
			last = next;
			double u = (double)rand() / (double)RAND_MAX;
			double X = -log(u)*mu;
			next = last + X;
		}
		
		return next;
	}

	double mu, last;
};

#endif