import os
import math
import numpy as np
import matplotlib as mpl
from matplotlib import rc
from matplotlib import pyplot as plt
from matplotlib.font_manager import FontProperties
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
rc('text', usetex=True)

mpl.rcParams['xtick.labelsize'] = 14
mpl.rcParams['ytick.labelsize'] = 14
mpl.rcParams['legend.fontsize'] = 10

scol = ['orange','cyan', 'magenta','brown','orange','green','black','darkviolet']
smark = ('x', 'o', '+', 's', '*', 'd', '^', 'p', '*', 'h', 'H', 'D', '8')

plt.close("all")

def FS_Efficiency(exp, xticks, xticksStr, xmin, xmax, ymin, ymax, x, leg):
    #prc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    prc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    grp2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    grp3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure(figsize=(5,4))

    #plt.semilogx(xticks, (prc2[:,6]*prc2[:,0]), linestyle='-', linewidth=2, label=r"Proc Sim. (n=2)")
    plt.plot(xticks, (grp2[:,6]), linestyle='-', linewidth=2, color='brown', label=r"Duplication Sim.")
    plt.plot(xticks, (prc3[:,6]), linestyle='-', linewidth=2, color='green', label=r"Proc Trip. Sim.")
    plt.plot(xticks, (grp3[:,6]), linestyle='-', linewidth=2, color='orange', label=r"Group Trip. Sim.")
    
    #plt.semilogx(xticks, (prc2[:,5]*prc2[:,0]), linestyle='--', linewidth=2, label=r"Proc Th. (n=2)")
    plt.plot(xticks, (grp2[:,5]), linestyle='--', linewidth=2, color='brown', label=r"Duplication Th.")
    plt.plot(xticks, (prc3[:,5]), linestyle='--', linewidth=2, color='green', label=r"Proc Trip. Th.")
    plt.plot(xticks, (grp3[:,5]), linestyle='--', linewidth=2, color='orange', label=r"Group Trip. Th.")

    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Efficiency')
    plt.xlim(xmin, xmax)
    plt.ylim(ymin, ymax)
    plt.subplots_adjust(left=0.12, bottom=0.12)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-E.pdf'))
    plt.draw()

def Mu_Efficiency(exp, xticks, xticksStr, xmin, xmax, ymin, ymax, x, leg):
    #prc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    prc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    grp2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    grp3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure(figsize=(5,4))

    #plt.semilogx(xticks, (prc2[:,6]*prc2[:,0]), linestyle='-', linewidth=2, label=r"Proc Sim. (n=2)")
    plt.semilogx(xticks, (grp2[:,6]), linestyle='-', linewidth=2, color='brown', label=r"Duplication Sim.")
    plt.semilogx(xticks, (prc3[:,6]), linestyle='-', linewidth=2, color='green', label=r"Proc Trip. Sim.")
    plt.semilogx(xticks, (grp3[:,6]), linestyle='-', linewidth=2, color='orange', label=r"Group Trip. Sim.")
    
    #plt.semilogx(xticks, (prc2[:,5]*prc2[:,0]), linestyle='--', linewidth=2, label=r"Proc Th. (n=2)")
    plt.semilogx(xticks, (grp2[:,5]), linestyle='--', linewidth=2, color='brown', label=r"Duplication Th.")
    plt.semilogx(xticks, (prc3[:,5]), linestyle='--', linewidth=2, color='green', label=r"Proc Trip. Th.")
    plt.semilogx(xticks, (grp3[:,5]), linestyle='--', linewidth=2, color='orange', label=r"Group Trip. Th.")

    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Efficiency')
    plt.xlim(xmin, xmax)
    plt.ylim(ymin, ymax)
    plt.subplots_adjust(left=0.12, bottom=0.12)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-E.pdf'))
    plt.draw()

def P_Efficiency(exp, xticks, xticksStr, xmin, xmax, ymin, ymax, x, leg):
    #prc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    prc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    grp2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    grp3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    mpl.rcParams['legend.numpoints'] = 1
    plt.figure(figsize=(5,4))

    #plt.semilogx(xticks, (prc2[:,6]*prc2[:,0]), linestyle='-', linewidth=2, label=r"Proc Sim. (n=2)")
    plt.plot(xticks, (grp2[:-1,6]), linestyle='-', linewidth=2, color='brown', label=r"Duplication Sim.")
    plt.plot(xticks[:-17], (prc3[:-18,6]), linestyle='-', linewidth=2, color='green', label=r"Proc Trip. Sim.")
    plt.plot(xticks[:-17], (grp3[:-18,6]), linestyle='-', linewidth=2, color='orange', label=r"Group Trip. Sim.")
    
    
    plt.plot((grp2[-1,3],grp2[-1,3]),(grp2[-1,6],grp2[-1,6]), marker='o', linestyle='None', linewidth=0, markersize=6, color='brown', label=r"Duplication Opt.")
    plt.plot((prc3[-1,3],prc3[-1,3]),(prc3[-1,6],prc3[-1,6]), marker='o', linestyle='None', linewidth=0, markersize=6, color='green', label=r"Proc Trip. Opt.")
    plt.plot((grp3[-1,3],grp3[-1,3]),(grp3[-1,6],grp3[-1,6]), marker='o', linestyle='None', linewidth=0, markersize=6, color='orange', label=r"Group Trip. Opt.")
    
    plt.tight_layout()
    #plt.xticks(xticks, xticksStr, rotation=0)
    plt.xticks([10000*i for i in range(0,50,5)], [r'$'+str(i)+'\cdot 10^4$' for i in range(0,51,5)], rotation=25)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Efficiency')
    plt.xlim(xmin, xmax)
    plt.ylim(ymin, ymax)
    plt.subplots_adjust(left=0.13, bottom=0.18)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-E.pdf'))
    plt.draw()
    mpl.rcParams['legend.numpoints'] = 2

def barplot(exp, save, v, xticks, xticksStr, xmin, xmax, ymin, ymax, x, y, leg):
    #prc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    prc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    grp2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    grp3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure(figsize=(5,4))

    if v == 3:
        prc3[:,v] = prc3[:,v]/1e5
        grp3[:,v] = grp3[:,v]/1e5
        grp2[:,v] = grp2[:,v]/1e5
        yticks = [i for i in range(0,15)]
        yticksStr = [r'$'+str(i)+r'\cdot 10^5$' for i in range(0,15)]

    if v == 4:
        prc3[:,v] = np.log10(prc3[:,v])
        grp3[:,v] = np.log10(grp3[:,v])
        grp2[:,v] = np.log10(grp2[:,v])
        yticks = [i for i in range(0,15)]
        yticksStr = [r'$10^{'+str(i)+r'}$' for i in range(0,15)]

    if v == 10:
        W = 1e6
        prc3[:,11] = (W / (prc3[:,0]*prc3[:,6]))*(prc3[:,11]/24)
        grp3[:,11] = (W / (grp3[:,0]*grp3[:,6]))*(grp3[:,11]/24)
        grp2[:,11] = (W / (grp2[:,0]*grp3[:,6]))*(grp2[:,11]/24)
        prc3[:,10] = (W / (prc3[:,0]*prc3[:,6]))*(prc3[:,10]/24)
        grp3[:,10] = (W / (grp3[:,0]*grp3[:,6]))*(grp3[:,10]/24)
        grp2[:,10] = (W / (grp2[:,0]*grp3[:,6]))*(grp2[:,10]/24)

    for i in range(0,len(xticks)):
        if v == 10:
            plt.bar(i+1-0.3-0.3/2, prc3[i,11], width=0.05, color='blue', edgecolor='none', label=r"Number of Errors" if i == 1 else '')
            plt.bar(i+1-0.3/2, grp3[i,11], width=0.05, color='blue', edgecolor='none', label='')
            plt.bar(i+1+0.3-0.3/2, grp2[i,11], width=0.05, color='blue', edgecolor='none', label='')
        plt.bar(i+1-0.3-0.3/2, prc3[i,v], width=0.3, linewidth=0.5, color='green', edgecolor='black', label=r"Proc Trip." if i == 1 else '')
        plt.bar(i+1-0.3/2, grp3[i,v], width=0.3, linewidth=0.5, color='orange', edgecolor='black', label=r"Group Trip." if i == 1 else '')
        plt.bar(i+1+0.3-0.3/2, grp2[i,v], width=0.3, linewidth=0.5, color='brown', edgecolor='black', label=r"Duplication" if i == 1 else '')
    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    if v == 3 or v == 4:
        plt.yticks(yticks, yticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel(y)
    plt.xlim(xmin, xmax)
    plt.ylim(ymin, ymax)
    plt.subplots_adjust(left=0.13, bottom=0.12)
    if v == 3:
        plt.subplots_adjust(left=0.16, bottom=0.12)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-'+save+'.pdf'))
    plt.draw()

# inputs = 'Mu-c1800-a1e6'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBE', 'upper right')
# barplot(inputs, 'nR', 10, xticksInt, xticksStr, 0, 6, 0, 250, 'System MTBE', 'Number of Recoveries per $10^6$ hours of work', 'upper left')
# barplot(inputs, 'T', 4, xticksInt, xticksStr, 0, 6, 1, 10, 'System MTBE', 'Optimal Pattern Length', 'upper right')
# barplot(inputs, 'P', 3, xticksInt, xticksStr, 0, 6, 0, 7, 'System MTBE', 'Optimal Number of processes', 'upper right')

# inputs = 'Mu-c60-a1e6'
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBE', 'lower left')
# barplot(inputs, 'nR', 10, xticksInt, xticksStr, 0, 6, 0, 250, 'System MTBE', 'Number of Recoveries per $10^6$ hours of work', 'upper left')
# barplot(inputs, 'T', 4, xticksInt, xticksStr, 0, 6, 1, 10, 'System MTBE', 'Optimal Pattern Length', 'upper right')
# barplot(inputs, 'P', 3, xticksInt, xticksStr, 0, 6, 0, 7, 'System MTBE', 'Optimal Number of processes', 'upper right')

# inputs = 'Mu-d1e7-a1e6'
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBE', 'lower left')
# barplot(inputs, 'nR', 10, xticksInt, xticksStr, 0, 6, 0, 250, 'System MTBE', 'Number of Recoveries per $10^6$ hours of work', 'upper left')
# barplot(inputs, 'T', 4, xticksInt, xticksStr, 0, 6, 1, 10, 'System MTBE', 'Optimal Pattern Length', 'upper right')
# barplot(inputs, 'P', 3, xticksInt, xticksStr, 0, 6, 0, 7, 'System MTBE', 'Optimal Number of processes', 'upper right')

# inputs = 'Mu-c1800-a1e7'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBE', 'lower left')
# barplot(inputs, 'P', 3, xticksInt, xticksStr, 0, 6, 0, 7, 'System MTBE', 'Optimal Number of processes', 'upper right')

# inputs = 'Mu-c1800-a1e5'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBE', 'upper right')
# barplot(inputs, 'P', 3, xticksInt, xticksStr, 0, 6, 0, 7, 'System MTBE', 'Optimal Number of processes', 'upper right')

# inputs = 'Mu2-P-a1e5'
# xticks = [10000*i for i in range(1,51)];
# xticksStr = [r'$'+str(i)+'\cdot 10^4$' for i in range(0,51)]
# P_Efficiency(inputs, xticks, xticksStr, 10000, 500000, 0, 0.1, 'Number of processes', 'upper left')

# inputs = 'Mu-P-a1e5'
# xticks = [10000*i for i in range(1,51)];
# xticksStr = [r'$'+str(i)+'\cdot 10^4$' for i in range(0,51)]
# P_Efficiency(inputs, xticks, xticksStr, 10000, 500000, 0, 0.1, 'Number of processes', 'upper left')

# inputs = 'FS-Mu-1e6'
# xticks = [i for i in range(0,101,10)];
# xticksStr = [r'$'+str(i)+'\%$' for i in range(0,101,10)]
# FS_Efficiency(inputs, xticks, xticksStr, 0, 100, 0, 0.5, 'Percentage of fail-stop errors', 'lower left')

# inputs = 'FS-Mu-1e5'
# xticks = [i for i in range(0,101,10)];
# xticksStr = [r'$'+str(i)+'\%$' for i in range(0,101,10)]
# FS_Efficiency(inputs, xticks, xticksStr, 0, 100, 0, 0.5, 'Percentage of fail-stop errors', 'lower left')

inputs = 'FS-Mu-1e3'
xticks = [i for i in range(0,101,10)]; xticksInt = [i for i in range(1,12)]
xticksStr = [r'$'+str(i)+'\%$' for i in range(0,101,10)]
FS_Efficiency(inputs, xticks, xticksStr, 0, 100, 0, 0.5, 'Percentage of fail-stop errors', 'upper left')
barplot(inputs, 'nR', 10, xticksInt, xticksStr, 0.25, 11.5, 0, 22, 'Percentage of fail-stop errors', 'Number of Recoveries per $10^6$ hours of work', 'upper left')

# inputs = 'FS-Mu-1e3'
# xticks = [i for i in range(0,101,10)];
# xticksStr = [r'$'+str(i)+'\%$' for i in range(0,101,10)]
# FS_Efficiency(inputs, xticks, xticksStr, 0, 100, 0, 0.5, 'Percentage of fail-stop errors', 'lower left')

# inputs = 'FS-Mu-fixed-1e6'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBF', 'lower left')

# inputs = 'FS-Mu-fixed-1e5'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBF', 'lower left')

# inputs = 'FS-Mu-fixed-1e4'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBF', 'lower left')

# inputs = 'FS-Mu-fixed-1e3'
# xticks = [1e6/10**i for i in range(0,5)]; xticksInt = [i for i in range(1,6)]
# xticksStr = [r'$10^{'+str((6-i))+'}$' for i in range(0,5)]
# Mu_Efficiency(inputs, xticks, xticksStr, 1e6, 1e2, 0, 0.5, 'System MTBF', 'lower left')

plt.show()