import os
import math
import numpy as np
import matplotlib as mpl
from matplotlib import rc
from matplotlib import pyplot as plt
from matplotlib.font_manager import FontProperties
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
rc('text', usetex=True)

mpl.rcParams['xtick.labelsize'] = 12
mpl.rcParams['ytick.labelsize'] = 12
mpl.rcParams['legend.fontsize'] = 12

scol = ['orange','cyan', 'magenta','blue','red','green','black','darkviolet']
smark = ('x', 'o', '+', 's', '*', 'd', '^', 'p', '*', 'h', 'H', 'D', '8')

plt.close("all")

def draw_H(exp, xticks, xticksStr, xmin, xmax, x, leg):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure()

    plt.plot(xticks, 1.0/(group2[:,6]*group2[:,0]), linestyle='-', linewidth=2, color='blue', label=r"Proc./Group. Sim. $(n=2)$")
    plt.plot(xticks, 1.0/(proc3[:,6]*proc3[:,0]), linestyle='-', linewidth=2, color='green', label=r"Proc. Sim. $(n=3)$")
    plt.plot(xticks, 1.0/(group3[:,6]*group3[:,0]), linestyle='-', linewidth=2, color='red', label=r"Group. Sim. $(n=3)$")

    #plt.plot(xticks, 1.0/(proc2[:,5]*proc2[:,0]), linestyle='--', linewidth=2, label=r"Proc. Th. $(n=2)$")
    #plt.plot(xticks, 1.0/(proc2[:,6]*proc2[:,0]), linestyle='-', linewidth=2, label=r"Proc. Sim. $(n=2)$")
    plt.plot(xticks, 1.0/(group2[:,5]*group2[:,0]), linestyle='--', linewidth=2, color='blue', label=r"Proc./Group. Th. $(n=2)$")
    plt.plot(xticks, 1.0/(proc3[:,5]*proc3[:,0]), linestyle='--', linewidth=2, color='green', label=r"Proc. Th. $(n=3)$")
    plt.plot(xticks, 1.0/(group3[:,5]*group3[:,0]), linestyle='--', linewidth=2, color='red', label=r"Group. Th. $(n=3)$")

    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Effiency')
    plt.xlim(xmin, xmax)
    plt.ylim(0,0.5)
    plt.subplots_adjust(left=0.09, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-H.pdf'))
    plt.draw()

def draw_log_H(exp, xticks, xticksStr, xmin, xmax, x, leg):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure()

    plt.semilogx(xticks, 1.0/(group2[:,6]*group2[:,0]), linestyle='-', linewidth=2, color='blue', label=r"Proc./Group. Sim. $(n=2)$")
    plt.semilogx(xticks, 1.0/(proc3[:,6]*proc3[:,0]), linestyle='-', linewidth=2, color='green', label=r"Proc. Sim. $(n=3)$")
    plt.semilogx(xticks, 1.0/(group3[:,6]*group3[:,0]), linestyle='-', linewidth=2, color='red', label=r"Group. Sim. $(n=3)$")

    #plt.plot(xticks, 1.0/(proc2[:,5]*proc2[:,0]), linestyle='--', linewidth=2, label=r"Proc. Th. $(n=2)$")
    #plt.plot(xticks, 1.0/(proc2[:,6]*proc2[:,0]), linestyle='-', linewidth=2, label=r"Proc. Sim. $(n=2)$")
    plt.semilogx(xticks, 1.0/(group2[:,5]*group2[:,0]), linestyle='--', linewidth=2, color='blue', label=r"Proc./Group. Th. $(n=2)$")
    plt.semilogx(xticks, 1.0/(proc3[:,5]*proc3[:,0]), linestyle='--', linewidth=2, color='green', label=r"Proc. Th. $(n=3)$")
    plt.semilogx(xticks, 1.0/(group3[:,5]*group3[:,0]), linestyle='--', linewidth=2, color='red', label=r"Group. Th. $(n=3)$")

    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Efficiency')
    plt.xlim(xmin, xmax)
    #plt.ylim(0,10)
    plt.subplots_adjust(left=0.09, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-H.pdf'))
    plt.draw()

def draw_T(exp, xticks, xticksStr, xmin, xmax, x, leg):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure()

    #plt.plot(xticks, proc2[:,4], linestyle='-', linewidth=2, label=r"Proc. Th. $(n=2)$")
    plt.plot(xticks, group2[:,4], linestyle='-', linewidth=2, color='blue', label=r"Proc./Group. Th. $(n=2)$")
    plt.plot(xticks, proc3[:,4], linestyle='-', linewidth=2, color='green', label=r"Proc. Th. $(n=3)$")
    plt.plot(xticks, group3[:,4], linestyle='-', linewidth=2, color='red', label=r"Group. Th. $(n=3)$")
    
    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Optimal Pattern Size')
    plt.xlim(xmin, xmax)
    #plt.ylim(0,10)
    plt.subplots_adjust(left=0.09, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-T.pdf'))
    plt.draw()

def draw_nR(exp, xticks, xticksStr, xmin, xmax, x, leg):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure()

    #plt.plot(xticks, (proc2[:,10]/proc2[:,4])*3600*24, linestyle='-', linewidth=2, label=r"Proc. Sim. $(n=2)$")
    plt.plot(xticks, (group2[:,10]/group2[:,4])*3600*24, linestyle='-', color='blue', linewidth=2, label=r"Proc./Group. Sim. $(n=2)$")
    plt.plot(xticks, (proc3[:,10]/proc3[:,4])*3600*24, linestyle='-', color='green', linewidth=2, label=r"Proc. Sim. $(n=3)$")
    plt.plot(xticks, (group3[:,10]/group3[:,4])*3600*24, linestyle='-', color='red', linewidth=2, label=r"Group. Sim. $(n=3)$")
    
    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Recoveries per day')
    plt.xlim(xmin, xmax)
    #plt.ylim(0,10)
    plt.subplots_adjust(left=0.09, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-nR.pdf'))
    plt.draw()

def draw_P(exp, xticks, xticksStr, xmin, xmax, x, leg):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    plt.figure()

    #plt.semilogx(xticks, proc2[:,3]*proc2[:,1], linestyle='-', markevery=1, linewidth=2, label=r"Proc. Opt. Number of Processes $(n=2)$")
    plt.semilogx(xticks, group2[:,3]*group2[:,1], linestyle='-', markevery=1, linewidth=2, color='blue', label=r"Group. Opt. Number of Processes $(n=2)$")
    plt.semilogx(xticks, proc3[:,3]*proc3[:,1], linestyle='-', markevery=1, linewidth=2, color='green', label=r"Proc. Opt. Number of Processes $(n=3)$")
    plt.semilogx(xticks, group3[:,3]*group3[:,1], linestyle='-', markevery=1, linewidth=2, color='red', label=r"Group. Opt. Number of Processes $(n=3)$")
    
    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    plt.legend(loc=leg)
    plt.xlabel(x)
    plt.ylabel('Number of Processes')
    plt.xlim(xmin, xmax)
    plt.ylim(0,1300000)
    plt.subplots_adjust(left=0.12, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-P.pdf'))
    plt.draw()

def draw_diff(exp, xticks, xticksStr, xmin, xmax, x):
    proc2 = np.loadtxt('results/proc/var_'+exp+'_n2_k2.out')
    proc3 = np.loadtxt('results/proc/var_'+exp+'_n3_k2.out')
    group2 = np.loadtxt('results/group/var_'+exp+'_n2_k2.out')
    group3 = np.loadtxt('results/group/var_'+exp+'_n3_k2.out')

    #plt.figure()
    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()

    ax1.plot(xticks, group2[:,7], linestyle='-', linewidth=2, color='blue', label=r"Proc./Group. Abs. Diff. $(n=2)$")
    ax1.plot(xticks, proc3[:,7], linestyle='-', linewidth=2, color='green', label=r"Proc. Abs. Diff. $(n=3)$")
    ax1.plot(xticks, group3[:,7], linestyle='-', linewidth=2, color='red', label=r"Group. Abs. Diff. $(n=3)$")

    #plt.plot(xticks, 1.0/(proc2[:,5]*proc2[:,0]), linestyle='--', linewidth=2, label=r"Proc. Th. $(n=2)$")
    #plt.plot(xticks, 1.0/(proc2[:,6]*proc2[:,0]), linestyle='-', linewidth=2, label=r"Proc. Sim. $(n=2)$")
    ax2.plot(xticks, group2[:,8], linestyle='--', linewidth=2, color='blue', label=r"Proc./Group. Rel. Diff. $(n=2)$")
    ax2.plot(xticks, proc3[:,8], linestyle='--', linewidth=2, color='green', label=r"Proc. Rel. Diff. $(n=3)$")
    ax2.plot(xticks, group3[:,8], linestyle='--', linewidth=2, color='red', label=r"Group. Rel. Diff. $(n=3)$")

    plt.tight_layout()
    plt.xticks(xticks, xticksStr, rotation=0)
    ax1.legend(loc='upper left')
    ax2.legend(loc='upper right')
    ax1.set_xlabel(x)
    ax1.set_ylabel('Absolute Difference')
    ax2.set_ylabel('Relative Difference')

    plt.xlim(xmin, xmax)
    #plt.ylim(0,10)
    plt.subplots_adjust(left=0.09, bottom=0.09)
    plt.savefig(os.path.normpath('figures/var_'+exp+'-diff.pdf'))
    plt.draw()

# Varying P
xticks = []
xticksStr = []
for i in range(2, 8):
    xticks.append(i)
    xticksStr.append(r'$10^{'+str(i)+'}$')

draw_H('Q_C30', xticks, xticksStr, 2, 7, 'Number of Processes', 'lower left')
draw_T('Q_C30', xticks, xticksStr, 2, 7, 'Number of Processes', 'upper left')
draw_nR('Q_C30', xticks, xticksStr, 2, 7, 'Number of Processes', 'upper left')
draw_diff('Q_C30', xticks, xticksStr, 2, 7, 'Number of Processes')

# Varying alpha
xticks = [0.0, 1e-8, 1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1e-1]
xticksStr = []
for i in xticks:
    xticksStr.append(r'$'+str(i)+'$')

draw_log_H('alpha', xticks, xticksStr, 0.0, 0.1, 'alpha', 'upper right')
draw_P('alpha', xticks,  xticksStr, 0.0, 0.1, 'alpha', 'upper right')

plt.show()
